function fnOut = latexport(fig, name)
% LATEXPORT
% LATEXPORT(FIGHANDLE)
% LATEXPORT(NAME)
% LATEXPORT(FIGHANDLE, NAME)
% 
% Generates a TeX file and an eps file, in a manner similar to
% Arno Linnemann's LAPRINT. There are a number of differences:
% 
%    (1) The LaTeX strings are stored in 'UserData' properties,
%        so that the MATLAB version of the figure still looks
%        intelligible. Thus, the 'String' properties can be set to
%        provide the nearest approximation that MATLAB can manage,
%        which means that the correct layout can be achieved more
%        easily, with a reasonable preview.
%    (2) Multi-line strings and strings with newline characters in
%        them are supported.
%    (3) Figure width and font scaling preferences are stored in
%        the figure's 'UserData' property. There is no GUI.
% 
% If FIGHANDLE is missing, the current figure is exported.
% If NAME is missing, the user is prompted.
% FIGHANDLE can be a list of figures provided NAME, if supplied,
% is a cell array of the corresponding size and shape.
% 
% The figure can be included in a LaTeX document with:
%   \begin{figure} \input{\figdir NAME.tex} \end{figure}
% 
% The following packages are required:
%   \usepackage{graphicx, psfrag}
% 
% The following macros must be pre-defined in the LaTeX document:
%   \newlength{\tempfigwidth} % will be set by the generated TeX file
%   \newlength{\tempfigheight} % will be set by the generated TeX file
% 
%   \newlength{\portraitfigwidth} \setlength{\portraitfigwidth}{....}
%   \newlength{\landscapefigheight} \setlength{\landscapefigheight}{....}
%       % two dimensions which determine the amount of horizontal space
%       % figures take up by default
% 
%   \newcommand{\figdir}{...}  % path to the directory where the two
% 	                           % generated files will reside.
%                              % (set to {} if in same dir as .tex source)
% 
% The LATEX command can be used to set the LaTeX alternatives for
% TEXT objects and axis tick labels, and to specify the following
% preferences for the figure as a whole:
% 
% 'Width'
%  default: '\portraitfigwidth' if portrait, '!' if landscape
%     The final figure should have this width (in cm if numeric).
% 	
% 'Height'
%  default: '!' if portrait, '\landscapefigheight' if landscape
%     The final figure should have this height (in cm if numeric).
% 
% 'BaseFontSizeOnScreen'
%  default: MATLAB's default text size
%     Match this font size on screen...
% 
% 'BaseFontSizeOnPaper'
%  default: 12
%     ...to this font size on paper, before scaling.
% 	
% 'FontScaling'
%  default: 0.8
%     Once font sizes are matched, scale them by this factor.
%     This is an aesthetic maneuver, cribbed from Arno Linneman,
%     whereby the base font size in a figure can be made smaller
%     than the base font size of the body text.
% 
% 'FontFamily'
%  default: 'cmss'
%     Specifies the font family to use for ordinary text ('cmr', 'cmss', etc).
%     To leave the font family unspecified in the LaTeX document, set
%     FontFamily to ''.
% 
% 'MarkerScaling'
%  default: 0.8
%     All marker sizes are scaled by this value on export.
% 
% See also LATEX

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

% requires: texprintf, writetext, makelegalname, latex, printfig

% permanent preferences
makenewfig = 0;
emulateTypeface = 1;
defaultFontSizeOnScreen = get(0, 'defaulttextfontsize');
defaultFontSizeOnPaper = 12;
defaultFontScalingFactor = 0.8;
defaultMarkerScalingFactor = 0.8;
defaultFontFamily = 'cmss';

if nargin < 2, name = ''; end
if nargin < 1, fig = get(0, 'currentfigure'); end
if isstr(fig) & isempty(name), name = fig; fig = get(0, 'currentfigure'); end
if isempty(fig), error('no figure'), end
fig = fig(:);
if ~all(ismember(fig, findobj(fig, 'type', 'figure')))
	error('inputs must be valid figure handles')
end
if length(fig) > 1
	if isempty(name)
		name = repmat({''}, size(fig));
	else
		name = cellstr(name); name = name(:);
		if length(unique(lower(name))) ~= length(fig)
			error('if names are supplied, one name per figure is required')
		end
	end
	for i = 1:length(fig), fn(i, :) = latexport(fig(i), name{i}); end
	if nargout, fnOut = fn; end
	return
end

figprefs = latex(fig);
widthStr = eval('figprefs.width', '[]');
heightStr = eval('figprefs.height', '[]');
fontScalingFactor = eval('figprefs.fontscaling', '[]');
baseFontSizeOnPaper = eval('figprefs.basefontsizeonpaper', '[]');
baseFontSizeOnScreen = eval('figprefs.basefontsizeonscreen', '[]');
fontFamily = eval('figprefs.fontfamily', '[]');
markerScalingFactor = eval('figprefs.markerscaling', '[]');

if isempty(fontFamily), fontFamily = defaultFontFamily; end
if isempty(baseFontSizeOnScreen), baseFontSizeOnScreen = defaultFontSizeOnScreen; end
if isempty(baseFontSizeOnPaper), baseFontSizeOnPaper = defaultFontSizeOnPaper; end
if isempty(fontScalingFactor), fontScalingFactor = defaultFontScalingFactor; end
if isempty(markerScalingFactor), markerScalingFactor = defaultMarkerScalingFactor; end
if isempty(widthStr), widthStr = '!'; end
if isempty(heightStr), heightStr = '!'; end
if isnumeric(widthStr), widthStr = sprintf('%gcm', widthStr); end
if isnumeric(heightStr), heightStr = sprintf('%gcm', heightStr); end
if strcmp(widthStr, '!') & strcmp(heightStr, '!')
	if strcmp(get(fig, 'paperorientation'), 'landscape')
		heightStr = '\landscapefigheight';
	else
		widthStr = '\portraitfigwidth';
	end
end

if nargout, fnOut = {'', ''}; end
if isempty(name)
	defaultName = get(fig, 'name');
	if isempty(defaultName), defaultName = sprintf('fig%g', fig); end
	try, name = epsname(fig); end
	if isempty(name), name = [makelegalname(defaultName) '.eps']; end
	[name parent] = uiputfile(name, sprintf('EPS file name for %s', defaultName));
	if isempty(parent) | ~isstr(parent) | ~isstr(name), return, end
	cd(parent), [ans stem ext] = fileparts(name);
	if isempty(ext), ext = '.eps'; end
	name = fullfile(parent, [stem ext]);
end	

[parent stem ext] = fileparts(name);
lastdot = max(findstr(ext, '.'));
if lastdot > 1, stem = [stem ext(1:lastdot-1)]; ext(1:lastdot-1) = []; end
if ~any(strcmpi(ext, {'.eps', '.tex'})), stem = [stem ext]; end
texfilename = [stem '.tex'];
epsfilename = [stem '.eps'];
fn{1} = fullfile(parent, texfilename);
fn{2} = fullfile(parent, epsfilename);
try, epsname(fig, epsfilename), end

if makenewfig
	fig = copyobj(fig, 0);
else
	savedFigPropNames = {'units', 'position', 'paperunits', 'paperposition', 'paperpositionmode', 'paperorientation'};
	savedFigPropValues = get(fig, savedFigPropNames);
end

set(fig, 'units', 'centimeters', 'paperunits', 'centimeters')
pos = get(fig, 'position');
set(fig, 'position', pos, 'paperposition', pos, 'paperorientation', 'portrait')
try, key('refresh'), end

texth = findall(fig, 'type', 'text', 'visible', 'on');
axh = findall(fig, 'type', 'axes', 'visible', 'on');
lineh = findall(fig, 'type', 'line', 'visible', 'on');
oldMarkerSizes = get(lineh, {'markersize'});
if ~isempty(oldMarkerSizes) & markerScalingFactor ~= 1
	set(lineh, {'markersize'}, num2cell(markerScalingFactor * [oldMarkerSizes{:}]'))
end

if ~makenewfig
	savedAxesPropNames = {'xticklabel', 'yticklabel', 'zticklabel', 'xticklabelmode', 'yticklabelmode', 'zticklabelmode', 'xtickmode', 'ytickmode', 'ztickmode'};
	savedAxesPropValues = get(axh, savedAxesPropNames);
	savedTextPropNames = {'units', 'fontunits', 'string'};
	savedTextPropValues = get(texth, savedTextPropNames);
end

frags = {};
for h = texth(:)'
	frags = ProcessText(h, 'string', frags, emulateTypeface);
end
for h = axh(:)'
	for dim = 'xyz'
		set(h, [dim 'tickmode'], 'manual')
		if strcmpi(get(h, [dim 'scale']), 'log') & strcmpi(get(h, [dim 'ticklabelmode']), 'auto')
			lab = get(h, [dim 'ticklabel']);
			prefix = repmat('10^', size(lab, 1), 1);
			prefix(all(isspace(lab), 2), :) = ' ';
			set(h, [dim 'ticklabel'], [prefix lab]);
		end
		frags = ProcessText(h, [dim 'ticklabel'], frags, emulateTypeface);
	end
end
s = '';
s = [s texprintf('\begin{psfrags}%%%\n')];
s = [s texprintf('\psfragscanon%%%\n')];
s = [s frags{:}];

[num denom] = rat(fontScalingFactor);

includeOpts = {'width=\tempfigwidth', ',', 'height=\tempfigheight'};
if strcmp(heightStr, '!')
	includeOpts(2:3) = [];
else
	s = [s texprintf('\setlength{\tempfigheight}{%s} \multiply\tempfigheight by %d \divide\tempfigheight by %d %%%\n', heightStr, denom, num)];
end
if strcmp(widthStr, '!')
	includeOpts(1:2) = [];
else
	s = [s texprintf('\setlength{\tempfigwidth}{%s} \multiply\tempfigwidth by %d \divide\tempfigwidth by %d %%%\n', widthStr, denom, num)];
end
s = [s texprintf('\resizebox{%s}{%s}{\includegraphics[%s]{\figdir %s}}%%%\n', widthStr, heightStr, [includeOpts{:}], epsfilename)];
s = [s texprintf('\end{psfrags}%%%\n')];

writetext(s, fn{1})
w1 = getwarnstate; warning off
printfig(fig, fn{2})
setwarnstate(w1)

if makenewfig
	delete(fig)
else
	set(texth, savedTextPropNames, savedTextPropValues)	
	set(axh, savedAxesPropNames, savedAxesPropValues)	
	set(fig, savedFigPropNames, savedFigPropValues)	
	if ~isempty(oldMarkerSizes) & markerScalingFactor ~= 1
		set(lineh, {'markersize'}, oldMarkerSizes)
	end
	try, key('refresh'), end
end


if nargout, fnOut = fn; end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function frags = ProcessText(h, prop, frags, emulateTypeface)

fontFamily = evalin('caller', 'fontFamily');
baseFontSizeOnScreen = evalin('caller', 'baseFontSizeOnScreen');
baseFontSizeOnPaper = evalin('caller', 'baseFontSizeOnPaper');

[str subwidth] = ResolveStringFormat(get(h, prop));
nlines = length(str);

alt = latex(h);
if isfield(alt, prop), alt = getfield(alt, prop); else alt = ''; end
if ~isempty(alt)
	str = ResolveStringFormat(alt);
	if length(str) ~= nlines
		set(h, 'color', [1 0 0], 'fontsize', 20)
		error(sprintf('handle %g has different numbers of lines in the MATLAB preview string and the LaTeX substitution string', h))
	end
end
eps = {};

if strcmpi(get(h, 'type'), 'text')
	set(h, 'units', 'points', 'fontunits', 'points')
	ext = get(h, 'extent'); ext = max(ext(3:4));
	equiv_length = max(1, round(1.6 * ext / get(h, 'fontsize')));
	subwidth = min(subwidth, equiv_length);
end
fmt = sprintf('%%0%dd', subwidth);


for i = 1:length(str)
	if isempty(str{i})
		eps{i} = '';
	else
		count = size(frags, 1) + 1;	
		eps{i} = sprintf(fmt, count);
		if strcmpi(prop, 'xticklabel')
			if strcmpi(get(h, 'xaxislocation'), 'top'), align = 'B'; else align = 't'; end
		elseif strcmpi(prop, 'yticklabel')
			if strcmpi(get(h, 'yaxislocation'), 'right'), align = 'l'; else align = 'r'; end
		else align = '';
		end
		frags{count, 1} = 	psfrag(eps{i}, str{i}, ~isempty(alt), h, align, emulateTypeface);
	end
end
set(h, prop, char(eps))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t, lenx] = ResolveStringFormat(s)
% S may be expressed as a cell array or a character matrix, and
% each cell (or row) may contain newline characters. End-of-cell,
% end-of-row, and newline character *all* delimit a new string
% element. Given this rule, return a cell array of the string
% elements of S.

nl = sprintf('\n');
s = char(s);
s = [s repmat(nl, size(s, 1), 1)];
s = s'; s = s(:)';
t = {}; lenx = [];
while ~isempty(s)
	i = min(findstr(s, nl));
	t{end+1, 1} = deblank(fliplr(deblank(fliplr(s(1:i-1)))));
	lenx(end+1) = length(t{end});
	s(1:i+length(nl)-1) = [];	
end
lenx = max(lenx);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function frag = psfrag(eps, tex, gotlatex, h, align, emulateTypeface)

fontFamily = evalin('caller', 'fontFamily');

if ~gotlatex
% If the user specifically supplied a LaTeX alternative, then
% gotlatex will be true, and it is up to the user to set LaTeX
% math mode explicitly or not, as appropriate.
% If the user supplied no LaTex explicitly, gotlatex is 0, and
% we need to use a few rules of thumb to decide whether or not
% to use math mode. These rules may miss, but the user can easily
% set a LaTeX alternative explicitly.	
	if any(ismember(tex, '\^_'))
%	If the string contains any of the TeX escape characters
% 	recognized by MATLAB, then set the whole string in math mode.
		tex = ['$' tex '$'];
	elseif ~isempty(fontFamily) & all(ismember(tex, '+-.0123456789'))
% 	If the string is a lone number, and a font family has been
% 	explictly specified, don't go into math mode. This ensures that
% 	numeric tick labels are set in that font family, rather than the
% 	default math mode family.
		tex = tex; % no change.
		if strncmp(tex, '-', 1) & ~strncmp(tex, '--', 2), tex = ['-' tex]; end
	else
% 	Otherwise, set the whole string in math mode if & only if
% 	there are no two consecutive letters.
		a = isletter(tex);
		consec = a(1:end-1) & ~diff(a);
		if ~any(consec), tex = ['$' tex '$']; end
	end
end

typeface = '';
if ~isempty(fontFamily), typeface = [typeface '\fontfamily{' fontFamily '}']; end

if emulateTypeface
% 	Attempt to emulate font size in a WYSIWYG manner
% 	(but a uniform size will probably look better in a LaTeX document).
	u = get(h, 'fontunits');
	set(h, 'fontunits', 'points')
	fontSizePts = get(h, 'fontsize');
	set(h, 'fontunits', u)

	parent = get(h, 'parent');
	while ~strcmpi(get(parent, 'type'), 'figure')
		parent = get(parent, 'parent');
	end
		
	u = get(parent, 'units');
	set(parent, 'units', 'centimeters')
	screenCoords = get(parent, 'position');
	set(parent, 'units', u)
	
	u = get(parent, 'paperunits');
	set(parent, 'paperunits', 'centimeters')
	paperCoords = get(parent, 'paperposition');
	set(parent, 'paperunits', u)

	baseFontSizeOnScreen = evalin('caller', 'baseFontSizeOnScreen');
	baseFontSizeOnPaper = evalin('caller', 'baseFontSizeOnPaper');
	siz = fontSizePts * baseFontSizeOnPaper / baseFontSizeOnScreen;
	typeface =  [typeface texprintf('\fontsize{%d}{%d}', round(siz), ceil(siz*1.2))];
%%
	switch lower(get(h, 'fontweight'))
		case 'light', typeface = [typeface '\fontseries{l}\mathversion{normal}'];
		case 'demi', typeface = [typeface '\fontseries{sb}\mathversion{bold}'];
		case 'bold', typeface = [typeface '\fontseries{bx}\mathversion{bold}'];
		otherwise ,typeface = [typeface '\fontseries{m}\mathversion{normal}'];
	end	
	switch lower(get(h, 'fontangle'))
		case 'italic', typeface = [typeface '\fontshape{it}'];
		case 'oblique', typeface = [typeface '\fontshape{it}'];
		otherwise, typeface = [typeface '\fontshape{n}'];
	end
end
if ~isempty(typeface), typeface = [typeface '\selectfont ']; end

if isempty(align)
	try, ha = lower(get(h, 'horizontalalignment')); catch, ha = 'center'; end
	try, va = lower(get(h, 'verticalalignment')); catch, va = 'middle'; end
	switch ha
		case 'left', align = [align 'l'];
		case 'right', align = [align 'r'];
	end
	switch va
		case 'top', align = [align 't'];
		case 'cap', align = [align 't'];
		case 'baseline', align = [align 'b'];
		case 'bottom', align = [align 'B'];
	end
end

frag = texprintf('\psfrag{%s}[%s][%s]{%s%s}%%%\n', eps, align, align, typeface, tex);
